<?php
namespace Teknisa\CUS\Modules\Relatorio\Services;

class Cus01600DesvioDeConsumoService {
    protected $newCode;
    protected $environment;
    protected $entityManager;
    protected $connection;

    public function __construct($environment, $entityManager, $newCode){
        $this->newCode        = $newCode;
        $this->environment    = $environment;
        $this->entityManager  = $entityManager;
        $this->connection     = $this->entityManager->getConnection();
    }

    /**
     * Função responsável por retornar um novo número de lista para ser usado.
     * 
     * @return {String} Número da lista gerada.
    */
    public function getNrLista(){
        return $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
    }
    
    /**
     * Função responsável por inserir um novo registro na tabela lista.
     * 
     * @param {String} $cdLista Dado a ser gravado na lista.
     * @param {String} $nrlista Número da lista a ser usada.
    */
    public function insertLista($cdlista, $nrlista) {

        $lista = new \Geral\Entities\Lista();
        $lista = $this->buildLista($lista, $nrlista, $cdlista);
        
        $this->entityManager->persist($lista);
        $this->entityManager->flush();
        
    }

    /**
     * Função responsável por remover um registro da tabela lista.
     * 
     * @param {String} $nrlista Dado a ser removido da lista.
    */
    public function removeLista($nrlista) {
        
        $params = array(
            'nrlista' => $nrlista,
            'nrorg'   => $this->environment->getNrOrg(),
        );
        
        $listas = $this->entityManager->getRepository('Geral\Entities\Lista')->findBy($params);
        
        foreach ($listas as $lista) {
            
            $this->entityManager->remove($lista);
            $this->entityManager->flush();
        }
    }

    /**
     *  Função responsável por retornar os serviços e filiais concatenados
     *  de acordo com parâmetros passados para serem colocados na tabela LISTA.
     *  Ex retorno: 999900001_ (Filial: 9999 Serviço: 00001)
     * 
     * @param {String} $$cdfilial String contendo os codigos
     *  das filiais concatenados com '_'.
     * 
     * @param {String} $cdgrupserv String contendo os codigos
     *  dos gropos de serviços concatenados com '_'.
     * 
     * @param {String} $cdservfili String contendo os codigos
     *  dos serviços concatenados com '_'.
    */
    public function getServico($nrListaFiliais, $cdgrupserv, $idutretiprat){

        $sql = "SELECT S.CDFILIAL||S.CDSERVFILI AS CDSERVFILI
                FROM   LISTA L, SERVICO S
                WHERE  L.NRLISTA    = :P_NRLISTAFILI AND
                       L.CDLISTA    = S.CDFILIAL AND
                      (INSTR(:P_CDGRUPSERV, S.CDGRUPSERV) <> 0  OR (:P_CDGRUPSERV = 'T')) AND
                      ((S.IDUTRETIPRAT = :P_UTRETIPRAT) OR (:P_UTRETIPRAT = 'T')) AND
                      ((S.IDTIPOSERV = 'N') OR (S.IDTIPOSERV = 'D'))
                ORDER BY S.CDFILIAL, S.CDSERVFILI";
        
        $params = array(
            "P_CDGRUPSERV" => $cdgrupserv,
            "P_NRLISTAFILI"   => $nrListaFiliais,
            "P_UTRETIPRAT" => $idutretiprat
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        $result = $query->fetchAll();

        // $result = implode('_', array_column($result, 'RESULT'));

        return array_column($result, 'CDSERVFILI');
    }
    
    /**
     * Função responsável por retornar as filiais que o operador logado tem acesso
     * de acordo com os parâmetros passados.
     * 
     * @param {String} $$cdfilial String contendo os codigos
     *  das filiais concatenados com '_'.
     * parametros operador logado e nrorg
    */    
    public function getFilial ($cdoperador, $cdgrupfili){
        
        $sql = "SELECT F.NRORG, F.CDFILIAL, F.NMFILIAL, '' AS CDGRUPFILI, F.IDCNTRRESC, F.CDREGIAO
             FROM FILIAL F, FILIOPER FO
             WHERE ( FO.CDFILIAL = F.CDFILIAL )
                 AND ( FO.CDOPERADOR =:CDOPERADOR )
                 AND :CDGRUPFILI = 'T'
             UNION
             SELECT DISTINCT F.NRORG, F.CDFILIAL, F.NMFILIAL, G.CDGRUPFILI, F.IDCNTRRESC, F.CDREGIAO
             FROM FILIAL F, FILGRUPFILI G, FILIOPER FO
             WHERE ( G.CDGRUPFILI =:CDGRUPFILI )
                 AND ( F.CDFILIAL = G.CDFILIAL )
                 AND ( FO.CDFILIAL = F.CDFILIAL )
                 AND ( FO.CDOPERADOR =:CDOPERADOR )";
        
        $params = array(
            "CDOPERADOR" => $cdoperador,
            "CDGRUPFILI" => $cdgrupfili
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        
        return array_column($result, 'CDFILIAL');
        
    }
}